// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// File        : winch_link.c
// Description : Linked lists service
// License     :
//
//  Copyright (C) 2019 Rachid Koucha <rachid dot koucha at gmail dot com>
//
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//
// Evolutions  :
//
//     31-Jul-2019  R. Koucha           - Creation
//
// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=


#include <assert.h>
#include <stdio.h>

#include "winch_link.h"





// ----------------------------------------------------------------------------
// Name   : winch_link_head
// Usage  : Link an element at the beginning of a double linked list
// Return : None
// ----------------------------------------------------------------------------
void winch_link_head(
		     winch_link_t *lk,   // Link
		     winch_link_t *l     // List
                    )
{
  assert(l);
  assert(lk);

  if (l->head)
  {
    assert(l->tail);
    lk->next = l->head;
    l->head->prev = lk;
    lk->prev = NULL;
    l->head = lk;
  }
  else  // Sole element in the list
  {
    assert(NULL == l->tail);
    l->head = l->tail = lk;
    lk->next = lk->prev = NULL;
  }

} // winch_link_head


// ----------------------------------------------------------------------------
// Name   : winch_link_tail
// Usage  : Link an element at the end of a double linked list
// Return : None
// ----------------------------------------------------------------------------
void winch_link_tail(
		     winch_link_t *lk,   // Link
		     winch_link_t *l     // List
                    )
{
  assert(l);
  assert(lk);

  if (l->tail)
  {
    assert(l->head);
    lk->next = NULL;
    assert(NULL == l->tail->next);
    l->tail->next = lk;
    lk->prev = l->tail;
    l->tail = lk;
  }
  else  // Sole element in the list
  {
    assert(NULL == l->head);
    l->head = l->tail = lk;
    lk->next = lk->prev = NULL;
  }

} // winch_link_tail


// ----------------------------------------------------------------------------
// Name   : winch_link_before
// Usage  : Link element 'lkb' before element 'lka' in list 'l'
// Return : None
// ----------------------------------------------------------------------------
void winch_link_before(
		       winch_link_t *lkb,   // Link before
		       winch_link_t *lka,   // Link after
		       winch_link_t *l      // List
                      )
{
  assert(l);
  assert(lka);
  assert(lkb);
  assert(NULL == lkb->next);
  assert(NULL == lkb->prev);

  // If head of list
  if (lka == l->head)
  {
    assert(NULL == lka->prev);
    l->head = lkb;
  }
  else
  {
    assert(lka->prev);
    lka->prev->next = lkb;
  }

  // If tail of list
  if (lka == l->tail)
  {
    assert(NULL == lka->next);
  }

  lkb->next = lka;
  lkb->prev = lka->prev;
  lka->prev = lkb;
} // winch_link_before


// ----------------------------------------------------------------------------
// Name   : winch_unlink
// Usage  : Unlink an element from the list
// Return : None
// ----------------------------------------------------------------------------
void winch_unlink(
		  winch_link_t *lk,   // Link
		  winch_link_t *l     // List
                 )
{
  assert(l);
  assert(lk);

  // If not the head of the list
  if (lk->prev)
  {
    assert(l->head != lk);
    lk->prev->next = lk->next;
  }
  else // It is the head of the list
  {
    assert(l->head == lk);
    l->head = lk->next;
  }

  // If not the tail of the list
  if (lk->next)
  {
    assert(l->tail != lk);
    lk->next->prev = lk->prev;
  }
  else // It is the tail of the list
  {
    assert(l->tail == lk);
    l->tail = lk->prev;
  }

  // The element is now out of the list
  lk->next = lk->prev = NULL;
} // winch_unlink

