// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// File        : winch_link.h
// Description : Linked lists service
// License     :
//
//  Copyright (C) 2019 Rachid Koucha <rachid dot koucha at gmail dot com>
//
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//
// Evolutions  :
//
//     31-Jul-2019  R. Koucha           - Creation
//
// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=




#ifndef WINCH_LINK_H
#define WINCH_LINK_H



// ----------------------------------------------------------------------------
// Name   : winch_link_t
// Usage  : Link descriptor
// ----------------------------------------------------------------------------
typedef struct winch_link
{
  struct winch_link *next; // Must be the 1st field to make calls to
                           // WINCH_LINK2...() work !
  struct winch_link *prev;
} winch_link_t;

// Defines for list's descriptor
#define head next
#define tail prev


#define winch_link_init(lk) ((lk)->next = (lk)->prev = (struct winch_link *)0)

#define winch_list_init(l) winch_link_init(l)

#define winch_is_empty_list(l) (!((l)->head))

#define winch_is_linked(lk, l) ((lk)->next || (lk)->prev || ((l)->head == (lk)))

extern void winch_unlink(
		       winch_link_t *lk,   // Link
		       winch_link_t *l     // List
		      );

extern void winch_link_before(
			    winch_link_t *lkb,   // Link before
			    winch_link_t *lka,   // Link after
			    winch_link_t *l      // List
			   );

extern void winch_link_tail(
			  winch_link_t *lk,   // Link
			  winch_link_t *l     // List
			 );

extern void winch_link_head(
			  winch_link_t *lk,   // Link
			  winch_link_t *l     // List
			 );



#endif // WINCH_LINK_H
