// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// File        : winch_msg.h
// Description : Message management
// License     :
//
//  Copyright (C) 2019 Rachid Koucha <rachid dot koucha at gmail dot com>
//
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//
// Evolutions  :
//
//     07-Aug-2019  R. Koucha           - Creation
//     11-Aug-2019  R. Koucha           - Added management of endianness
//                                        and CPU dependant data sizes
//                                        (serialization over the network)
//
// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=



#ifndef WINCH_MSG_H
#define WINCH_MSG_H

#include <sys/types.h>



// ----------------------------------------------------------------------------
// Name   : WINCH_MSG_...
// Usage  : Message types
// ----------------------------------------------------------------------------
#define WINCH_MSG_WINSZ   0
#define WINCH_MSG_CMD     1


// ----------------------------------------------------------------------------
// Name   : WINCH_MSG_TYPE_FIELD
// Usage  : Type of the header fields on host
// ----------------------------------------------------------------------------
#define WINCH_MSG_TYPE_FIELD_TYPE int
#define WINCH_MSG_SIZE_FIELD_TYPE size_t


// ----------------------------------------------------------------------------
// Name   : winch_recv_msg
// Usage  : Receive a message from the network
// Return : 0, if OK
//          -1, if error
// ----------------------------------------------------------------------------
extern int winch_recv_msg(
                          int      fd,
                          WINCH_MSG_TYPE_FIELD_TYPE     *type,
                          WINCH_MSG_SIZE_FIELD_TYPE  *data_sz,
                          char   **data    // Dynamically allocated
                         );


// ----------------------------------------------------------------------------
// Name   : winch_send_msg
// Usage  : Send a message over the network
// Return : 0, if OK
//          -1, if error
// ----------------------------------------------------------------------------
extern int winch_send_msg(
                          int            fd,
                          WINCH_MSG_TYPE_FIELD_TYPE            type,
                          WINCH_MSG_SIZE_FIELD_TYPE         data_sz,
                          unsigned char *data
                         );

// ----------------------------------------------------------------------------
// Name   : winch_msg_init
// Usage  : Initialize the network service
// Return : None
// ----------------------------------------------------------------------------
extern void winch_msg_init(void);


// ----------------------------------------------------------------------------
// Name   : WINCH_MSG_HTON_16
// Usage  : Translate a 16 bits unsigned int on host into a table of 2 chars
//          to be transfered onto the network (big endian format) 
// Return : None
// ----------------------------------------------------------------------------
extern void WINCH_MSG_HTON_16(uint16_t v, unsigned char *bytes);


// ----------------------------------------------------------------------------
// Name   : WINCH_MSG_HTON_32
// Usage  : Translate a 32 bits unsigned int on host into a table of 4 chars
//          to be transfered onto the network (big endian format) 
// Return : None
// ----------------------------------------------------------------------------
extern void WINCH_MSG_HTON_32(uint32_t v, unsigned char *bytes);


// ----------------------------------------------------------------------------
// Name   : WINCH_MSG_NTOH_16
// Usage  : Translate 2 chars from the network (big endian format) into a
//          16 bits unsigned integer on host 
// Return : None
// ----------------------------------------------------------------------------
extern uint16_t WINCH_MSG_NTOH_16(unsigned char *bytes);

// ----------------------------------------------------------------------------
// Name   : WINCH_MSG_NTOH_32
// Usage  : Translate 4 chars from the network (big endian format) into a
//          32 bit unsigned integer on host 
// Return : None
// ----------------------------------------------------------------------------
extern uint32_t WINCH_MSG_NTOH_32(unsigned char *bytes);


#endif // WINCH_MSG_H
